#
# Copyright (c) 2004 Art Haas
#
# This file is part of PythonCAD.
# 
# PythonCAD is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# PythonCAD is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with PythonCAD; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#
# _The_ base class for all objects used in PythonCAD
#

import message

class Entity(message.Messenger):
    messages = {
        'locked' : True,
        'unlocked' : True,
        'modified' : True,
        'reset' : True, 
        'reparented' : True,
        'hidden' : True,
        'exposed' : True,
        'added_child' : True,
        'removed_child' : True,
        }
    def __init__(self, parent=None):
        message.Messenger.__init__(self)
        self.__locked = False
        self.__modified = False
        self.__visible = True
        if parent is not None:
            if not isinstance(parent, Entity):
                raise TypeError, "Invalid parent: " + `parent`
        self.__parent = parent
        self.__children = None

    def isLocked(self):
        return self.__locked
    
    def lock(self):
        if not self.__locked:
            self.__locked = True
        self.sendMessage('locked')

    def unlock(self):
        if self.__locked:
            self.__locked = False
        self.sendMessage('unlocked')

    def isModified(self):
        return self.__modified

    def modified(self):
        if not self.__modified:
            self.__modified = True
        self.sendMessage('modified') # always send this message

    def reset(self):
        if self.__modified:
            self.__modified = False
        self.sendMessage('reset') # always send this message

    def isVisible(self):
        return self.__visible

    def hide(self):
        if self.__visible:
            self.__visible = False
        self.sendMessage('hidden')

    def show(self):
        if not self.__visible:
            self.__visible = True
        self.sendMessage('exposed')

    def expose(self):
        Entity.show(self)
        
    def setParent(self, parent):
        _oldparent = self.__parent
        if parent is not None:
            if not isinstance(parent, Entity):
                raise TypeError, "Invalid parent: " + `parent`
        if _oldparent is not parent:
            if _oldparent is not None:
                _oldparent.delChild(self)
            self.__parent = parent
            if parent is not None:
                parent.addChild(self)
            self.sendMessage('reparented', _oldparent)

    def getParent(self):
        return self.__parent

    def addChild(self, child):
        if not isinstance(child, Entity):
            raise TypeError, "Invalid child: " + `child`
        if child.getParent() is not self:
            raise ValueError, "Invalid parent: " + `child`
        if self.__children is None:
            self.__children = []
        if child in self.__children:
            raise ValueError, "Child entity already stored: " + `child`
        self.__children.append(child)
        self.sendMessage('added_child', child)

    def delChild(self, child):
        if not isinstance(child, Entity):
            raise TypeError, "Invalid child: " + `child`
        if child.getParent() is not self:
            raise ValueError, "Invalid parent for child: " + `child`
        if self.__children is None:
            raise ValueError, "Entity has no children: " + `self`
        if child not in self.__children:
            raise ValueError, "Child entity not stored: " + `child`
        self.__children.remove(child)
        if len(self.__children) == 0:
            self.__children = None
        self.sendMessage('removed_child', child)
        
    def getChildren(self):
        if self.__children is not None:
            return self.__children[:]
        return []
    
    def sendsMessage(self, m):
        if m in Entity.messages:
            return True
        return message.Messenger.sendsMessage(self, m)

    def clone(self):
        return Entity(self.__parent)
    
    def inRegion(self, xmin, ymin, xmax, ymax, all=False):
        return False
        
class F1(object):
    keys = {
        'a' : True,
        'b' : True,
        'c' : True
        }
    def __init__(self):
        pass

    def getValue(self, key):
        return F1.keys.get(key)
    
class F2(F1):
    keys = {
        'd' : True,
        'e' : True
        }
    def __init__(self):
        pass

    def getValue(self, key):
        _val = F2.keys.get(key)
        if _val is None:
            _val = F1.getValue(self, key)
        return _val

    def blah(self):
        print "F2::blah()"
        
